# This file is part of "TW-Light" 
#                     http://tw-light.berlios.de/
# Copyright (C) 2001-2005  TimeWarp development team

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

#  Module API:
#  * runDialogString(str_xml) 
#  * runDialogFile(file_path)

import unittest, os, random, re
import xml.parsers.expat

""" For testing purpose we import fake TML implementation that provide the same API as
    real TML, but doesn't contain real game code."""
if __name__ == '__main__':
    import tml_fake as tml
else:
    import tml
if __name__ == '__main__':
    import misc_fake as misc
else:
    import misc

import quests
from flags import *

random.seed()
g_cur_dialog = ""

def runDialogString(str_xml):
    DialogLoader().loadDialogString(str_xml).doDialog()
def runDialogFile(file_path):
    if os.access(file_path, os.F_OK):
        DialogLoader().loadDialogFile(file_path).doDialog()

#########################################################
# Native dialog API
#########################################################
def goto(id):
    g_cur_dialog.goto(id)

def endDialog():
    g_cur_dialog.endDialog()    
#########################################################
# Helpers functions

def get_text(text):
    tup = text.split('|')
    return re.sub("{(.*?)}", lambda m: eval(m.group(1)), random.choice(tup))

#########################################################
class Local:
    def __init__(self):
        pass
local = Local()

class Action:
    def __init__(self,cond,eval):
        self.cond = cond     # should be true for evaluating action
        self.eval = eval     # code to evaluate
    def evaluate(self):
        if self.checkCond():
            exec(self.eval)
    def checkCond(self):    # return true if condition is true, false otherwise
        if eval(self.cond):
            return True
        return False
    
class PlayerAnswer(Action):
    def __init__(self,text,appear_condition,actions):
        self.text         = text              # player answer text
        self.cond         = appear_condition  # if true variant visible
        self.actions      = actions           # actions tuple that should run on evaluate
        
    def evaluate(self):
        for act in self.actions:
            act.evaluate()

# Maing dialog unit
class Node:
    def __init__(self, id, name, music, background, alien_text, font, text_location, player_answers):
        self.id             = id                     # node id must be unique within dialog
        self.name           = name                   # human readable node name, for debug output 
        self.music          = music                  # file path to music 
        self.background     = background             # file path to background image
        self.alien_text     = alien_text             # alien text delimited with <br>
        self.font           = font                   # font for dialog
        self.text_location  = text_location          # tuple (x, y) x,y from 0..100% of the screen
        self.player_answers = player_answers         # player Answers tuple
    def evaluate(self):
        tml.DialogApi.showAlienPicture(self.background)
        tml.DialogApi.showText(get_text(self.alien_text), 0, self.text_location[0], self.text_location[1])
        tml.GetSoundSystem().play_music(self.music)
        answers = filter(lambda x: x.checkCond(), self.player_answers)
        answ_texts = []
        for answer in answers:
            answ_texts.append(get_text(answer.text))
        answers[tml.DialogApi.askPlayer(misc.PyListStringToVector(answ_texts))].evaluate()
        
class Dialog:
    def __init__(self, startcond = (), endcond = (), nodeset = None):
        self.nodeset  = nodeset     # tuple with all dialog data, terminated by None object
        self.cur_node = nodeset[0]  # currently selectet node
        global g_cur_dialog
        g_cur_dialog = self
        global local
        local = Local()
        self.start_cond = startcond
        self.end_cond   = endcond
        for act in self.start_cond:
            act.evaluate()
    def doDialog(self): # run dialog untill selected node != None
        global g_cur_dialog
        g_cur_dialog = self
        tml.DialogApi.startDialog()
        while(self.cur_node != None):
            self.cur_node.evaluate()
        tml.DialogApi.endDialog()
    def goto(self,id): # switch current node
        self.cur_node = filter(lambda x: x.id == id, self.nodeset)[0]
    def endDialog(self):
        for act in self.end_cond:
            act.evaluate()
        self.cur_node = None
        
class DialogLoader:
    """ This class can be used to transform dialog editor's XML files to Dialog objects

        Generaly you should only call one of functions
          * loadDialogString ()
          * loadDialogFile ()
    """
    def __init__(self):
        self.p = xml.parsers.expat.ParserCreate()
        self.p.StartElementHandler    = self.start_element
        self.p.EndElementHandler      = self.end_element
        self.python_code = ""
    def loadDialogString(self, dlg):
        self.python_code = ""
        self.p.Parse(dlg, 1)
        return eval(self.python_code)
    def loadDialogFile(self, dlg):
        self.python_code = ""
        f = open(dlg,'r')
        self.p.ParseFile(f)
        f.close()
        return eval(self.python_code)
    #####################################################################
    def start_element(self, name, attrs):
        startMethod = getattr(self, 'start' + name)
        self.python_code += startMethod( name,attrs )
    def end_element(self, name):
        endMethod = getattr(self, 'end' + name)
        self.python_code += endMethod( name )
    ######################################################################
    def startACTION(self, name, attrs):
        return 'Action("""' + attrs['condition'] + '""","""'  + attrs['action'] + '"""),'
    def startPLAYER_ANSWER(self, name, attrs):
        return 'PlayerAnswer("""' + attrs['text'] + '""","""' + attrs['appear_condition'] + '""",('
    def startNODE(self, name, attrs):
        return 'Node(' + attrs['id'] + ',"""' + \
               attrs['name'] + '""","""' + \
               attrs['music'] + '""","""' + \
               attrs['background'] + '""","""' \
               +  attrs['alien_text'] + '""","""' + \
               attrs['font'] + '""",' + \
               attrs['text_location'] + ',('
    def startNODES(self, name, attrs):
        return 'nodeset = ('
    def startDIALOG(self, name, attrs):
        return "Dialog( "
    def startSTART_DIALOG(self, name, attrs):
        return " startcond = ("
    def startEND_DIALOG(self, name, attrs):
        return " endcond = ("
    def startSTART_NODES(self, name, attrs):
        return ""
    def startSTART_NODE(self, name, attrs):
        return ""
    def endPLAYER_ANSWER(self,name):
        return ')),'
    def endNODE(self,name):
        return ')),'
    def endNODES(self,name):
        return '))'
    def endACTION(self, name):
        return ""
    def endDIALOG(self, name):
        return ""
    def endSTART_DIALOG(self, name):
        return "), "
    def endEND_DIALOG(self, name):
        return "), "
    def endSTART_NODES(self, name):
        return ""
    def endSTART_NODE(self, name):
        return ""


#####################################################################################################
#                              TESTS                                                                #
#####################################################################################################
class TestActionFunctions(unittest.TestCase):
    def setUp(self):
        global test_global_value
        test_global_value = 0
    def tearDown(self):
        pass
    def testCheckCond(self):
        action = Action('1', 'pass')
        self.assert_(action.checkCond())
        action = Action('0', 'pass')
        self.assert_(not action.checkCond())
    def testEvaluate(self):
        action = Action('1', 'global test_global_value\ntest_global_value=25')
        action.evaluate()
        self.assertEqual(25, test_global_value)
        action = Action('0', 'global test_global_value\ntest_global_value=14')
        action.evaluate()
        self.assertNotEqual(14, test_global_value)

class TestPlayerAnswerFunctions(unittest.TestCase):
    def setUp(self):
        global test_global_value
        test_global_value = 0
        pass
    def tearDown(self):
        pass
    def testCheckCond(self):
        playerAnswer = PlayerAnswer('Text', '1', ())
        self.assert_(playerAnswer.checkCond())
        playerAnswer = PlayerAnswer('Text', '0',())
        self.assert_(not playerAnswer.checkCond())
    def testEvaluate(self):
        playerAnswer = PlayerAnswer('Text', '1', (Action('1', 'global test_global_value\ntest_global_value=15'),
                                                  Action('1==0', 'global test_global_value\ntest_global_value=14')))
        playerAnswer.evaluate()
        self.assertEqual(15, test_global_value)

class TestDialogFunctions(unittest.TestCase):
    start_cond = ()
    end_cond   = ()
    nodeset = (
        Node(0, 'Node1','','','AlienText','',(50,10),
             (PlayerAnswer('Answer1', '1', (Action('1', 'goto(1)'), )),
              PlayerAnswer('Answer2', '1', (Action('1', 'endDialog()'),)
                           )))
        ,
        Node(1, 'Node1','','','AlienText','',(50,10),
             (PlayerAnswer('Answer3', '1', (Action('1', 'endDialog()'),)
                           ),)),
        ) 
    def make_dialog(self, start_cond, end_cond, nodeset):
        return Dialog(start_cond, end_cond, nodeset)
    
    def setUp(self):
        tml.DialogApi.clearJornal()
        self.dialog = self.make_dialog( self.start_cond, self.end_cond, self.nodeset)
    def testGoto(self):
        self.dialog.goto(1)
        self.assertEqual(self.dialog.cur_node.id,1)
    def testEndDialog(self):
        self.dialog.endDialog()
        self.assertEqual(self.dialog.cur_node,None)
    def testDoDialog(self):
        self.dialog.doDialog()
        self.assertEqual(tml.DialogApi.getJornal(),"startDialog showAlienPicture showText play_music askPlayer showAlienPicture showText play_music askPlayer endDialog ")
    def testStartCondition(self):
        self.dialog = self.make_dialog( (Action('1', 'goto(1)'),), self.end_cond, self.nodeset)
        self.dialog.doDialog()
        self.assertEqual(tml.DialogApi.getJornal(),"startDialog showAlienPicture showText play_music askPlayer endDialog ")
        pass
    def testUsingLocalVar(self):
        self.dialog = self.make_dialog( (Action('1', 'goto(1);local.abc=1'),), self.end_cond, self.nodeset)
        self.assertEqual(1, local.abc)
        pass
    
class TestDialogLoaderFunctions(unittest.TestCase):
    def setUp(self):
        tml.DialogApi.clearJornal()
    def testAction(self):
        dlg_ldr = DialogLoader()
        dlg_ldr.start_element('ACTION', {'action' : 'goto(2)','condition' : '1'})
        self.assertEqual('Action("""1""","""goto(2)"""),',dlg_ldr.python_code)
    def testPlayerAnswer(self):
        dlg_ldr = DialogLoader()
        dlg_ldr.start_element('PLAYER_ANSWER', {'appear_condition' : '1','text' : 'we come in peace'})
        self.assertEqual('PlayerAnswer("""we come in peace""","""1""",(',dlg_ldr.python_code)
    def testPlayerEndElements(self):
        dlg_ldr = DialogLoader()
        dlg_ldr.end_element('PLAYER_ANSWER')
        self.assertEqual(')),',dlg_ldr.python_code)
        dlg_ldr = DialogLoader()
        dlg_ldr.end_element('NODE')
        self.assertEqual(')),',dlg_ldr.python_code)
        dlg_ldr = DialogLoader()
        dlg_ldr.end_element('NODES')
        self.assertEqual('))',dlg_ldr.python_code)

    def testNode(self):
        dlg_ldr = DialogLoader()
        dlg_ldr.start_element('NODE', {'text_location' : '(50,10)',
                                       'alien_text' : 'TW-Dialog',
                                       'background' : 'a.png',
                                       'id'         : '0',
                                       'name'       : 'Start',
                                       'font'       : 'Times',
                                       'music'      : 'a.ogg'})
        self.assertEqual('Node(0,"""Start""","""a.ogg""","""a.png""","""TW-Dialog""","""Times""",(50,10),(',dlg_ldr.python_code)
    def testNodes(self):
        dlg_ldr = DialogLoader()
        dlg_ldr.start_element('NODES', {})
        self.assertEqual('nodeset = (',dlg_ldr.python_code)
    def testComplexTest(self):
        jornalString = "startDialog showAlienPicture showText play_music askPlayer showAlienPicture showText play_music askPlayer endDialog "
        f = open('test_dialog.xml','r')
        str = f.read()
        f.close()
        DialogLoader().loadDialogString(str).doDialog()
        self.assertEqual(tml.DialogApi.getJornal(),jornalString)
        tml.DialogApi.clearJornal()
        DialogLoader().loadDialogFile('test_dialog.xml').doDialog()
        self.assertEqual(tml.DialogApi.getJornal(),jornalString)
    def testStartCondition(self):
        dlg = DialogLoader().loadDialogFile('test_dialog.xml')
        self.assertEqual(len(dlg.start_cond),2)
        pass
    def testStartCondition(self):
        dlg = DialogLoader().loadDialogFile('test_dialog.xml')
        self.assertEqual(len(dlg.end_cond),1)
        pass

class TestGetAlienTextFunction(unittest.TestCase):
    def testSimpleRandom(self):
        text = "Hi|Hello"
        res = get_text(text)
        self.assert_(res == "Hi" or res == "Hello")
    def testSimpleRandomEmpty(self):
        self.assertEqual(get_text(""),"")
    def testTextWithCalledCode(self):
        global test_player_name
        test_player_name = "Yura"
        text = "hi, {test_player_name}!"
        self.assertEqual(get_text(text),"hi, Yura!")
    def testTextWith2CalledCode(self):
        global test_player_name
        test_player_name = "Yura"
        text = "hi, {test_player_name} {test_player_name}!"
        self.assertEqual(get_text(text),"hi, Yura Yura!")

if __name__ == '__main__':
    unittest.main()


        
