/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2004  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#include "util/sound.h"

#include <allegro.h>
#include <string.h>
#include <stdio.h>

#include "melee.h"
#include "id.h"
#include "frame.h"

#include "mcbodies.h"
#include "mgame.h"
#include "mcontrol.h"
#include "mship.h"
#include "mshot.h"
#include "manim.h"
#include "mview.h"
#include "other/twconfig.h"
#include "scp.h"
#include "mitems.h"
#include "tml/gameconfig.h"
#include "tml/eventmanager.h"
#include "tml/gameevent.h"

#include "util/errors.h"
#include "WarpOutTrail.h"


/**
 * These are colours used when the ship warps out.  They are in the 
 * inverse order as the ones used to warp in.
 */ 
int inverse_hot_color[INVERSE_HOT_COLORS] =
  {
    47, 47, 45, 44, 43, 42,
    127, 126, 125, 124, 123, 122,
    123, 124, 125, 126, 127,
    42,  43,  44,  45,  46,  47
  };



WarpOutTrail::WarpOutTrail(
	       SpaceLocation *creator, Vector2 opos, Vector2 _rpos, 
	       Ship *ship, SpaceSprite *sprite, int osprite_index, int *ocolors, 
	       int onum_colors, int ofsize, int steps, int step_size,
	       WarpEffect * effect) :
  SpaceObject(creator, opos, 0.0, sprite),
  rel_pos(_rpos),
  ship(ship),
  sprite_index(osprite_index),
  colors(ocolors),
  num_colors(onum_colors),
  color_index(0),
  frame_size(ofsize),
  frame_step(ofsize),
  phaser_step_position(0),
  phaser_steps(steps),
  phaser_step_size(step_size),
  effect(effect)
{
  STACKTRACE;
  layer = LAYER_HOTSPOTS;
  set_depth(DEPTH_HOTSPOTS);
  collide_flag_anyone = 0;
  mass = 0;

  attributes |= ATTRIB_UNDETECTABLE;
  
  // extra check
  // note that if this happens, there's something wrong in the ships' constructor...
  if (sprite_index >= sprite->frames())
  {
	  tw_error("!!!!");
	  sprite_index = 0;
  }
    
  if (effect)
     effect->startEffect();
}

void WarpOutTrail::animate(Frame *space) {
  STACKTRACE;
  sprite->animate_character(pos, 
			    sprite_index, pallete_color[colors[color_index]], space);
}

void WarpOutTrail::calculate() {
  STACKTRACE;
  frame_step -= frame_time;

  while (frame_step < 0) {
    frame_step += frame_size;
    color_index++;
    if (color_index == num_colors)
      state = 0;
  }

  if (phaser_step_position < phaser_step_size) 
  {
    phaser_step_position += frame_time;
    if (phaser_step_position >= phaser_step_size) 
	{
      if (phaser_steps > 1) 
	  {
         Vector2 d = rel_pos / phaser_steps;
         game->add(new WarpOutTrail(this, pos + d, rel_pos-d, ship, sprite, 
                sprite_index, colors, num_colors, frame_size, phaser_steps-1, 
                phaser_step_size, effect));
      } 
	  else 
	  {
		if (effect)
		{
			effect->endEffect();
			effect = NULL;
		}
      }
    }
  }
  SpaceObject::calculate();
  //die();
}

/*void WarpOutTrail::startEffect() {
  STACKTRACE;
  //destroy the ship
  //ship->state = 0;
//  tw_error("warpouttrail::startEffect()");
  printf("warpouttrail::startEffect()\n");
  
}

void WarpOutTrail::endEffect() {
  STACKTRACE;
  //tw_error("warpouttrail::endEffect()");
  printf("warpouttrail::endEffect()\n");
}*/


/*WarpOutTrail * WarpOutTrail::getWarpOutTrail(Ship * ship) {
  STACKTRACE;
  if (!ship) {
  	tw_error("In getWarpOutTrail(), parameter ship was null");
  	return NULL;
  }
  return new WarpOutTrail(ship,
		    ship->pos + unit_vector(ship->get_angle()) * 0 * ship->get_size().x,
		    unit_vector(ship->get_angle()) * PHASE_MAX * ship->get_size().x,
		    ship, 
		    ship->get_sprite(), 
		    ship->get_sprite_index(), 
		    inverse_hot_color, 
		    INVERSE_HOT_COLORS,
		    PHASE_DELAY, 
		    PHASE_MAX, 
		    PHASE_DELAY
		    );
}*/
