/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2004  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#ifndef WARPOUTTRAIL_HEADER
#define WARPOUTTRAIL_HEADER

#include "melee.h" // for hot_color
#include "mframe.h"


#define INVERSE_HOT_COLORS 23

/**
 * These are colours used when the ship warps out.  They are in the 
 * inverse order as the ones used to warp in.
 */ 
extern int inverse_hot_color[];


class WarpEffect {
public:
  /**
   * This method is called at the beginning of the effect.  The default
   * behaviour is to immediately remove the ship from play by setting its state
   * to 0.
   * Subclasses should override this method to customize behaviour; it is not 
   * necessary to call the parent's startEffect method.
   */ 
  virtual void startEffect() = 0;
  
  /**
   * This method is called at the end of the effect.  The default behaviour is
   * to do nothing; it is not necessary to call the parent's startEffect method.
   * Override this method to customize behaviour.  
   */ 
  virtual void endEffect() = 0;
};

/**
 * This creates a warp-in animated effect that lasts a few milliseconds; after
 * which, the ship is considered in play.
 * 
 * The visual effect is that the ship is warping into Truespace from Hyperspace.
 * The ship's outline is drawn as a trail in red-ish yellow colour.  
 */
class WarpOutTrail : public SpaceObject 
{
 protected:
  Vector2 rel_pos;
  Ship *ship;
  int sprite_index;
  int *colors;
  int num_colors;
  int color_index;
  int frame_size;
  int frame_step;
  int phaser_step_position;
  int phaser_steps;
  int phaser_step_size;
  
  WarpEffect * effect;
  
 public:
  /**
   * Removes a ship from play and displays a warp-out effect.  At the 
   * beginning of the effect, the ship is not longer in play.  The effect
   * lasts a few milliseconds afterwards.  
   * 
   * Typically, this constructor is called with the method
   * Ship::get_ship_phaser()
   * 
   * @see WarpOutTrail::getWarpOutTrail()
   *  
   * @param creator 
   * @param opos 
   * @param _rpos 
   * @param ship 
   * @param sprite 
   * @param osprite_index 
   * @param ocolors 
   * @param onum_colors 
   * @param ofsize 
   * @param steps 
   * @param step_size 
   */
  WarpOutTrail(SpaceLocation *creator, Vector2 pos, Vector2 rel_pos, 
		 Ship *ship, SpaceSprite *sprite, int osprite_index, int *ocolors, 
		 int onum_colors, int ofsize, int steps, int step_time,
		 WarpEffect * effect);
		 
  /**
   * Removes a ship from play and displays a warp-out effect.  At the 
   * beginning of the effect, the ship is not longer in play.  The effect
   * lasts a few milliseconds afterwards.  
   * 
   * This constructor makes the trail appear in front of the ship, lasts a fair
   * distance, and cycles the colours in an attractive pattern.  
   * 
   * This constructor is probably easier to work with than the other one for
   * this class.
   *  
   * @param ship 
   */
  //WarpOutTrail(Ship * ship);
  
  /**
   * Draws the warp-out effect
   * 
   */
  virtual void animate(Frame *space);

  /**
   * Updates the WarpOutTrail's status by:
   * * Cycling the ship's warp trail colour
   * * Moving the drawn effect along its vector
   * * If there are remaining steps to draw, a new WarpOutTrail is created to continue
   *   the remaining steps
   */
  virtual void calculate();
  
  
  /**
   * Creates a new warp-out effect for this ship.  At the start of the effect,
   * the ship is removed from play.  
   * @param ship the ship to warp out and remove from play
   */ 
  //static WarpOutTrail * getWarpOutTrail(Ship * ship);
};

#endif
