/* $Id: nullphas.h,v 1.1.1.1 2004/08/01 10:21:24 Yura Exp $ */ 
/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2004  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#include <iostream>

#include <allegro.h>
#ifdef WIN32
#include <allegro/platform/aintwin.h>
#include <winalleg.h>
#endif

#include "melee.h"
#include "frame.h"
#include "melee/mview.h"
#include "melee/mgame.h"
#include "scp.h"

#include "gamedialog.h"
#include "gameconfig.h"
#include "util/sound.h"

#define DIALOG_HEIGHT_AREA 0.7

static std::string g_current_music = "";
static bool g_paused = false;
void DialogApi::startDialog()
{
  STACKTRACE;
  std::cerr<<__FUNCTION__<<"\n";
  g_paused = game->is_paused();
  if(!game->is_paused())
    game->pause();
  g_current_music  = tw_sound->playing_music_file();
}

void DialogApi::endDialog()
{
  STACKTRACE;
  std::cerr<<__FUNCTION__<<"\n";
  if(!g_paused)
    game->unpause();
  tw_sound->play_music(g_current_music);
}

int DialogApi::askPlayer(const std::vector<std::string>& answers)
{
  STACKTRACE;
  std::cerr<<__FUNCTION__<<"\n";
  MENU *pMenuAnswers = new MENU[answers.size()+1];
  for( unsigned int i=0; i<answers.size(); i++)
    {
      pMenuAnswers[i].text   = strdup(answers[i].c_str());
      pMenuAnswers[i].proc   = NULL;
      pMenuAnswers[i].child  = NULL;
      pMenuAnswers[i].flags  = 0;
      pMenuAnswers[i].dp     = NULL;
    }
  pMenuAnswers[answers.size()].text   = NULL;
  pMenuAnswers[answers.size()].proc   = NULL;
  pMenuAnswers[answers.size()].child  = NULL;
  pMenuAnswers[answers.size()].flags  = 0;
  pMenuAnswers[answers.size()].dp     = NULL;
  
  show_mouse(game->window->surface);
  int ret = -1;
  while( ret < 0 )
    {
      ret = do_menu(pMenuAnswers, 0, (int)(game->view->frame->window->h*DIALOG_HEIGHT_AREA));
    }
  show_mouse(NULL);

  ///////////////////////////////////////////////
  //Free memory
  for(unsigned int i=0;i<answers.size();i++)
    free(pMenuAnswers[i].text);
  delete[] pMenuAnswers;
  return ret;
}
  
/*! \brief split string by words
	\param source string
	\chars maximum length of returned string
	\return result string
*/
static std::string SplitString(std::string source, unsigned int chars)
{
	std::string strResult;
	if (source.length()<chars+1)
		return source;

	int size = source.rfind(' ', chars);
	source.resize(size+1);
	return source;
}

void DialogApi::showText(const std::string& text, int delay, int x, int y)
{
  STACKTRACE;
  std::cerr<<__FUNCTION__<<"\n";
  RGB tmpPal[256];
  memcpy(tmpPal, palette_color, sizeof(RGB) * 256);
  set_palette((const RGB*)g_game_data[TW_FONT_PALETTE].dat);

  game->window->lock();


  ///
  std::string strText = text;
  std::string strTemp;

  int k = 0;
  while(strText.length() != 0 )
  {
	  std::string strTmp;
	  strTemp = SplitString(strText, 40);
	  textout_centre(game->view->frame->window->surface, 
			 D_FONT_ENGLISH, 
			 strTemp.c_str(), 
			 (int)(game->view->frame->window->w*x/100),  
			 (int)(game->view->frame->window->h*DIALOG_HEIGHT_AREA*y/100)+k, -1);
	  strText.erase(strText.begin(), strText.begin() + strTemp.length());
	  k+=text_height(D_FONT_ENGLISH) + 3;
  }
  game->window->unlock();
  videosystem->set_palette(tmpPal);
  HAVE_A_WAIT(delay);
}
 
void DialogApi::showAlienPicture(const std::string& path)
{
  STACKTRACE;
  std::cerr<<__FUNCTION__<<"\n";
  std::string pic = GameConfig::GetAbsolutePath(path);
  if(!pic.length())
    {
      tw_error( (std::string("Unable to find file: ") + path).c_str());
    }
  BITMAP* alienPic = load_bitmap(pic.c_str(), NULL);
  
  BITMAP* d_buf = create_bitmap(game->view->frame->window->w,
				game->view->frame->window->h);   
  clear_bitmap(d_buf);
  stretch_blit(alienPic, d_buf, 0, 0, alienPic->w, alienPic->h, 0, 0, d_buf->w, (int)(d_buf->h *DIALOG_HEIGHT_AREA));
  
  game->window->lock();
  blit(d_buf, 
       game->view->frame->window->surface, 
       0, 0, 
       game->view->frame->window->x, 
       game->view->frame->window->y, 
       d_buf->w, d_buf->h);
  game->window->unlock();

  destroy_bitmap(alienPic);
  destroy_bitmap(d_buf);
};
