/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2005  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#include <iostream>

#include "eventmanager.h"
#include "gameevent.h"
#include "games/ggob.h"
#include "python/python_class.h"
#include "melee.h"
#include "melee/mship.h"
#include "melee/mcontrol.h"

static int generateEventID()
{
  static int id = 0;
  id++;
  return id;
}

TGameEvent::TGameEvent()
{
  STACKTRACE;
  exist = true;
  id = generateEventID();
}

TGameEvent::~TGameEvent()
{

}

void TGameEvent::trigger()
{
  STACKTRACE;
  // execute function
  python::exec_function(_script);
}

bool TGameEvent::check_python_cond()
{
  for(std::set<std::string>::iterator i = m_condition.begin(); i!=m_condition.end();i++)
    {
      if(!python::exec_function(*i))
	return false;
    }
  return true;
}

void TOneTimeEvent::trigger()
{
  STACKTRACE;
  TOneTimeEventBase::trigger();
  die();
}

TEventTime::TEventTime()
{
  STACKTRACE;
}

TEventTime::~TEventTime()
{
}

bool TEventTime::calculate()
{
  STACKTRACE;
  if( _spot_time == gobgame->game_time / 1000 && check_python_cond())
    {
      trigger();
      return true;
    }
  return false;
}

int TEventTime::setUp(int time, const std::string& script)
{
  STACKTRACE;
  _spot_time = time;
  _script = script;
  return id;
}

TEventSingleTime::TEventSingleTime()
{
  STACKTRACE;  
}

TEventSingleTime::~TEventSingleTime()
{
}

bool TEventSingleTime::calculate()
{
  STACKTRACE;
  if(TEventTime::calculate() && check_python_cond() )
    {
      die();
      return true;
    }
  return false;
}

TEventRepeatableTime::TEventRepeatableTime()
{
  STACKTRACE;
  interval = -1;
}

TEventRepeatableTime::~TEventRepeatableTime()
{
}

bool TEventRepeatableTime::calculate()
{
  STACKTRACE;
  if(TEventRepeatableTimeBase::calculate() && check_python_cond())
    {
      _spot_time += interval;
      return true;
    }
  return false;
}

int TEventRepeatableTime::setUp(int time, const std::string& script)
{
  STACKTRACE;
  interval = time;
  TEventRepeatableTimeBase::setUp(gobgame->game_time/1000 + time, script);
  return id;
}


TEventShipDeathById::~TEventShipDeathById()
{
}

int TEventShipDeathById::setUp(const std::string& script)
{
  STACKTRACE;
  _script = script;
  return id;
}


TEventShipDeathByType::TEventShipDeathByType()
{
  STACKTRACE;
}
TEventShipDeathByType::~TEventShipDeathByType()
{
}

int TEventShipDeathByType::setUp(const std::string type, const std::string& script)
{
  STACKTRACE;
  _type = type;
  _script = script;
  return id;
}

bool TEventShipDeathByType::calculate(Ship* s)
{
  STACKTRACE;
  if(_type == s->type->id && check_python_cond() )
    {
      trigger();
      return true;
    }
  return false;
}


TEventDistance::TEventDistance()
{
  STACKTRACE;
}
TEventDistance::~TEventDistance()
{
}


bool TEventDistance::calculate()
{
  STACKTRACE;
  // check if location doesn't exist, event is invalid
  if(!m_location->exists()||!m_location2->exists())
    {
      die();
      return false;
    }
  return check();
}

bool TEventDistance::check_less()
{
  STACKTRACE;
  if(gobgame->in_itemP(m_location)  && 
     //gobgame->in_itemP(m_location2) && 
     m_location2->distance(m_location)<m_dist &&
     check_python_cond())
    {
      trigger();
      return true;
    }
  return false;
}

bool TEventDistance::check_more()
{
  STACKTRACE;
  if(gobgame->in_itemP(m_location)  && 
     //gobgame->in_itemP(m_location2) && 
     m_location2->distance(m_location)>m_dist&&
     check_python_cond())
    {
      trigger();
      return true;
    }
  return false;
}



TEventDistanceMoreThen1Obj::TEventDistanceMoreThen1Obj()
{
  STACKTRACE;
  m_location2 = new Asteroid();
}

TEventDistanceMoreThen1Obj::~TEventDistanceMoreThen1Obj()
{
  delete m_location2;
}


bool TEventDistanceMoreThen1Obj::check()
{
  STACKTRACE;
  return check_more();
}

int TEventDistanceMoreThen1Obj::setUp(SpaceLocation* l, Vector2 pos, double dist, const std::string& script)
{
  STACKTRACE;
  m_location = l;
  m_location2->pos = Vector2(0,0);
  m_location2->translate(pos);
  m_dist     = dist;
  _script    = script;
  return id;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
TEventDistanceLessThen1Obj::TEventDistanceLessThen1Obj()
{
  STACKTRACE;
}

TEventDistanceLessThen1Obj::~TEventDistanceLessThen1Obj()
{
}

bool TEventDistanceLessThen1Obj::check()
{
  STACKTRACE;
  return check_less();
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEventDistanceMoreThen2Obj::TEventDistanceMoreThen2Obj()
{
  STACKTRACE;
}
TEventDistanceMoreThen2Obj::~TEventDistanceMoreThen2Obj()
{
}

bool TEventDistanceMoreThen2Obj::check()
{
  STACKTRACE;
  return check_more();
}

int TEventDistanceMoreThen2Obj::setUp(SpaceLocation* l1, SpaceLocation* l2, double dist, const std::string& script)
{
  STACKTRACE;
  m_location  = l1;
  m_location2 = l2;;
  m_dist     = dist;
  _script    = script;
  return id;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEventDistanceLessThen2Obj::TEventDistanceLessThen2Obj()
{
  STACKTRACE;
}

TEventDistanceLessThen2Obj::~TEventDistanceLessThen2Obj()
{
}

bool TEventDistanceLessThen2Obj::check()
{
  STACKTRACE;
  return check_less();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////


TEventMoveTerminator::TEventMoveTerminator()
{
  STACKTRACE;
}
TEventMoveTerminator::~TEventMoveTerminator()
{
}

void TEventMoveTerminator::trigger()
{
  STACKTRACE;
  die();
}

void TEventMoveTerminator::die()
{
  STACKTRACE;
  TEventMoveTerminatorBase::die();
  EventManager::disableEvent(m_term_event);
}

int TEventMoveTerminator::setUp(SpaceLocation* l1, SpaceLocation* l2, double dist, int event_to_terminate)
{
  STACKTRACE;
  m_term_event = event_to_terminate;
  return TEventMoveTerminatorBase::setUp(l1,l2, dist, "");
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////


TEventOrderMoveToLocation::TEventOrderMoveToLocation()
{
  STACKTRACE;
}

TEventOrderMoveToLocation::~TEventOrderMoveToLocation()
{
  if(m_terminator && m_terminator->exists())
    {
      m_terminator->die();
    }
}

void TEventOrderMoveToLocation::die()
{
  STACKTRACE;
  if(m_terminator && m_terminator->exists())
    {
      m_terminator->die();
    }
  TEventOrderMoveToLocationBase::die();
}

bool TEventOrderMoveToLocation::calculate()
{
  STACKTRACE;
  if(!m_location->exists()||!m_location2->exists())
    {
      die();
      return true;
    }

  Ship* shp = (Ship*)m_location;
  if(shp->death_counter!=-1)
    {
      die();
      return true;  
    }

  
  if(shp->control->planet_dangerP())
    return false;
  
  // reset the keys for this ship
  shp->nextkeys = 0;

  // re-define the keys
  double a, da, db, dc;
  a = min_delta(m_location2->pos - shp->pos, map_size).atan();
  
  // direction
  da = a - shp->vel.atan();
  while (da < -PI)	da += PI2;
  while (da >  PI)	da -= PI2;
  
  db = a - shp->angle;
  while (db < -PI)	db += PI2;
  while (db >  PI)	db -= PI2;
  
  dc = shp->vel.atan() - shp->angle;
  while (dc < -PI)	dc += PI2;
  while (dc >  PI)	dc -= PI2;

  // don't turn so quickly that you'd come at a complete stop (180 degr)
  // but, some anti-thrust can be healthy otherwise you may keep circling?
  if (fabs(dc) > 0.6 * PI)
    da = dc;
  else
    {
      a = fabs(da/PI);
      if (a > 0.5) a = 0.5;
      da = a*da + (1-a)*db;
      // for a really big velocity deviation, attention goes to correcting that;
      // otherwise, the ship's direction should be focus
    }
  
  if (da > 0.01*PI)
    {
      shp->nextkeys |= keyflag::right;
    }
  if (da < -0.01*PI)
    {
      shp->nextkeys |= keyflag::left;
    }
  shp->nextkeys |= keyflag::thrust;	// you _must_ thrust, otherwise you can't steer.
  return false;
};

int TEventOrderMoveToLocation::setUp(SpaceLocation* l1, SpaceLocation* l2, double dist)
{
  STACKTRACE;
  TEventMoveTerminator* te = new TEventMoveTerminator();
  m_terminator = boost::shared_ptr<TGameEvent>(te);
  EventManager::g_events.push_back(m_terminator);
  TEventOrderMoveToLocationBase::setUp(l1, l2, dist, "");
  return te->setUp(l1, l2, dist, getID());
}


////////////////////////////////////////////////////////////////////////////////////////

TEventOrderMoveToPoint::TEventOrderMoveToPoint()
{
  STACKTRACE;
  m_location2 = new Asteroid();
}

TEventOrderMoveToPoint::~TEventOrderMoveToPoint()
{
  delete m_location2;
}

int TEventOrderMoveToPoint::setUp(SpaceLocation* l1, Vector2 pos, double dist)
{
  STACKTRACE;
  m_location2->pos = Vector2(0,0);
  m_location2->translate(pos);
  return TEventOrderMoveToPointBase::setUp(l1, m_location2, dist);
}
 

///////////////////////////////////////////////////////////////////////////////////////////////////////
TEventGuardLocation::TEventGuardLocation()
{
  STACKTRACE;
}

TEventGuardLocation::~TEventGuardLocation()
{
  if(m_move_to_location && m_move_to_location->exists())
    {
      m_move_to_location->die();
    }
}

void TEventGuardLocation::trigger()
{
  STACKTRACE;
  if( (bool)m_move_to_location && m_move_to_location->exists())
    return;
  
  TEventOrderMoveToLocation* te = new TEventOrderMoveToLocation();
  
  double dist = 0;
  if(m_dist>2000)
    {
      dist = m_dist - 500;
    }
  else
    {
      dist = 3*m_dist/4;
    }
  te->setUp(m_location, m_location2, dist);
  m_move_to_location = boost::shared_ptr<TGameEvent>(te);
  EventManager::g_events.push_back(m_move_to_location);
}

int TEventGuardLocation::setUp(SpaceLocation* l1, SpaceLocation* l2, double dist)
{
  STACKTRACE;
  return TEventGuardLocationBase::setUp(l1, l2, dist, "");
}

//////////////////////////////////////////////////////////////////////////////////////////////////
TEventGuardPoint::TEventGuardPoint()
{
  STACKTRACE;
   m_location2 = new Asteroid();
}
TEventGuardPoint::~TEventGuardPoint()
{
  delete m_location2;
};

int TEventGuardPoint::setUp(SpaceLocation* l1, Vector2 pos, double dist)
{
  STACKTRACE;
  m_location2->pos = Vector2(0,0);
  m_location2->translate(pos);
  return TEventGuardPointBase::setUp(l1, m_location2, dist);
}
////////////////////////////////////////////////////////////////////////////////////////////////////

bool TEventJumpToSystem::calculate()
{
	if(game->_name==m_destSystem)
	{
		trigger();
		return true;
	}
	return false;
}

int TEventJumpToSystem::setUp(const std::string& system, const std::string& script)
{
	_script      = script;
	m_destSystem = system;
	return id;
}



